__author__ = "Erre Quadro srl"
__email__ = "paolo.arduin@errequadrosrl.com"
__license__ = "Apache Software License 2.0"
__summary__ = "SPIKEX - SpaCy Pipes for Knowledge Extraction"
__title__ = "spikex"
__version__ = "0.3.0"
